
package mijdbc;

import java.io.*;

/**
 * JDBC Interface to Illustra functions
 * 
 * This class implements a BufferedOutputStream that creates a hexdump
 * when the buffer is flushed. This is useful to verify what is sent
 * to the illustra server.
 */

public class MIBufferedOutputStream extends BufferedOutputStream
{
  
  
  public MIBufferedOutputStream(OutputStream out, int size) {
    super(out, size);
  }
  
  
  
  /** Flush the internal buffer */
  private void flushBuffer() throws IOException {
    if (count > 0) {
      if (MId.dump) 
	DumpBuffer();
      out.write(buf, 0, count);
      count = 0;
    }
  }
  
  
  /**
   * Writes the specified byte to this buffered output stream. 
   *
   * @param      b   the byte to be written.
   * @exception  IOException  if an I/O error occurs.
   * @since      JDK1.0
   */
  public synchronized void write(int b) throws IOException {
    if (count >= buf.length) {
      flushBuffer();
    }
    buf[count++] = (byte)b;
  }
  

  /**
   * Flushes this buffered output stream. This forces any buffered 
   * output bytes to be written out to the underlying output stream. 
   *
   * @exception  IOException  if an I/O error occurs.
   * @see        java.io.FilterOutputStream#out
   */
  public synchronized void flush() throws IOException {
    flushBuffer();
    out.flush();
  }

  
  private void DumpBuffer() {
    int cnt = count;
    int pos = 0;

    System.out.println("Writing:");

    while (cnt > 0) {
      
      int n = cnt;
      if (n > 10)
	n = 10;

      MIHexDump.DumpBuffer(buf, pos, n);
      
      pos += n;
      cnt -= n;

    }
  }
  
}


