// Java Interface to Illustra
// Erwin Kalvelagen, TNO FEL

// 03/28/96 B. McLean

// Copyright (c) 1996 Bradley McLean / Jeffrey Medeiros
// Modifications Copyright (c) 1996/1997 Martin Rode
// Copyright (c) 1997 Peter T Mount
//
// You may distribute under the terms of the GNU Public License as
// specified in the README file that comes with the JavaPostgres95 Kit

package mijdbc;

import java.sql.*;
import java.util.*;

/**
 * JDBC Interface to Illustra functions
 */
 
public class MIDriver implements Driver
{
  // PM 13 May 97 As from now, these will reflect the version of the driver
  static final int MAJORVERSION = 0;
  static final int MINORVERSION = 1;

  // defaults
  static final String DEFAULT_PORT = "7531";
  static final String DEFAULT_HOST = "localhost";

  
  static java.util.Vector connections = new Vector();
  
  // PM 13 May 97 I have been informed that the driver should register
  // itself when the class is loaded (ie done in a static {} segment).
 
  static {
    try {
      DriverManager.registerDriver(new MIDriver());
    } catch(Exception e) {
    }
  }
  
  // Each Driver should also support a null constructor so it can be
  // instantiated by doing
  //
  //	java.sql.Driver d = Class.forName("foo.bah.Driver").newInstance();
  //
  // As the driver is already registered, we don't need to do anything here.
  //
  public MIDriver() throws SQLException
  {
  }
  
  // Try to make a database connection to the given URL.
  // The driver should return "null" if it realizes it is the wrong kind
  // of driver to connect to the given URL.  This will be common, as when
  // the JDBC driver manager is asked to connect to a given URL it passes
  // the URL to each loaded driver in turn.
  //
  // The driver should raise a SQLException if it is the right 
  // driver to connect to the given URL, but has trouble connecting to
  // the database.
  //
  // The java.util.Properties argument can be used to passed arbitrary
  // string tag/value pairs as connection arguments.
  // Normally at least a "user" and  "password" properties should be
  // included in the Properties.
  
  public Connection connect(String url, java.util.Properties info) throws SQLException
  {
    MIConnection MIC;
    
    /*
     * Decompose the url, and verify that it's for us.           
     */
	 
    String host = null;
    String port = null;
    String dbname = null;
    int index = url.indexOf(':');
    
    if( index == -1 )
      return null;	// illegal URL
    
    if( url.substring(0,index).compareTo("jdbc") != 0 )
      return null;	// wrong type of URL
    
    url = url.substring(index+1);
    
    index = url.indexOf(':');
    if( index == -1 )
      return null;	// illegal format
    
    if( url.substring(0,index).compareTo("mijdbc") != 0 )
      return null;	// wrong type of database
    
    url = url.substring(index+1);
    if( url.substring(0,2).compareTo("//") == 0 ) {
      url = url.substring(2);
      index = url.indexOf('/');
      if ( index == -1 )
	host = url;
      else {
	host = url.substring(0,index);
	dbname = url.substring(index+1);
      }
      index = host.indexOf(':');
      if ( index != -1 ) {
	port = host.substring(index+1);
	host = host.substring(0,index);
      }
    } else
      dbname = url;
    
    // Further parsing of database name here in future?
    // (Pull the options / username, etc out of the url?
    
    // Start with any environmental defaults
    String mihost = info.getProperty("MIHOST");
    String miport = info.getProperty("MIPORT");
    String mioptions = info.getProperty("MIOPTIONS");
    String mitty = info.getProperty("MITTY");
    String miuser = info.getProperty("user");
    String mipass = info.getProperty("password");
    
    // Plug in available info from the url.    
    if( host != null )
      mihost = host;
    
    if( port != null )
      miport = port;
    
    // And use hard defaults for anything not defined.
    if( mihost == null )
         mihost = DEFAULT_HOST;
    
    if( miport == null )
      miport = DEFAULT_PORT;
    
    if( mioptions == null )
      mioptions = "";
    
    if( mitty == null )
      mitty = "";
    
    if( dbname == null )
      return null;	// Should we throw an exception here?
    
    MIC = new MIConnection(this,mihost,miport,mioptions,mitty,dbname,miuser,mipass);
    
    // Keep track of our Connections
    connections.addElement((Object) MIC);
    
    return MIC;
  }
  
  void closeConnection(MIConnection child)
  {
    connections.removeElement((Object) child);
  }
  
  // getMajorVersion and getMinorVersion return the major and minor
  // versions of the driver.  Initially these should be 1 and 0.
  public int getMajorVersion()
  {
    return MAJORVERSION;
  }
  
  public int getMinorVersion()
  {
    return MINORVERSION;
  }
  
  public boolean acceptsURL(String url) throws SQLException 
  {
    return false;
  }
  
  public DriverPropertyInfo[] getPropertyInfo(String url, java.util.Properties info) throws SQLException 
  {
    return(null);   
  }
  
  // For now this returns false
  public boolean jdbcCompliant()
  {
    return false;
  }
  
} 
