// MIInflaterInputStream.java
//
//
// We overwrite a few methods from java.util.zip.InflaterInputStream.
// This in order to deal with a continuous stream of zipped blocks.
// 

package mijdbc;

import java.util.zip.*;
import java.io.*;

/**
 * JDBC Interface to Illustra functions
 * 
 */

public class MIInflaterInputStream extends InflaterInputStream
{

    /**
     * Creates a new input stream with a default decompressor and buffer size.
     * EK: Create a larger buffer!
     */
    public MIInflaterInputStream(InputStream in) {
	super(in, new Inflater(),8192);
    }

 
   /**
     * Reads uncompressed data into an array of bytes. This method will
     * block until some input can be decompressed.
     * @param b the buffer into which the data is read
     * @param off the start offset of the data
     * @param len the maximum number of bytes read
     * @return the actual number of bytes read, or -1 if the end of the
     *         compressed input is reached or a preset dictionary is needed
     * @exception ZipException if a ZIP format error has occurred
     * @exception IOException if an I/O error has occurred
     *
     * EK: handle inf.finished: prepare for next zipped block.
     */
    public int read(byte[] b, int off, int len) throws IOException {
	if (len <= 0) {
	    return 0;
	}
	try {
	    int n;
	    while ((n = inf.inflate(b, off, len)) == 0) {
	      if (inf.finished()) {

		// find out number of remaining bytes in buffer
		int remaining = inf.getRemaining();
		//System.out.println("Finished. Reset inflater. Remaining "+remaining);            
		inf.reset(); 
		if (remaining > 0) // restart with these remaining bytes
		  inf.setInput(buf, this.len-remaining, remaining);
		return read(b,off,len);
	      }
	      if (inf.needsDictionary()) {
		return -1;
	      }
	      if (inf.needsInput()) {
		fill();
	      }
	    }
	    return n;
	} catch (DataFormatException e) {
	    String s = e.getMessage();
	    throw new ZipException(s != null ? s : "Invalid ZLIB data format");
	}
    }



}
