// Java Interface to Illustra
// Erwin Kalvelagen, TNO FEL

// Copyright (c) 1996 Bradley McLean / Jeffrey Medeiros
// Modifications Copyright (c) 1996/1997 Martin Rode
// Copyright (c) 1997 Peter T Mount
//
// You may distribute under the terms of the GNU Public License as
// specified in the README file that comes with the JavaPostgres95 Kit

package mijdbc;

import java.sql.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * JDBC Interface to Illustra functions
 */

class MINotify {
   int pid;
   String name;
   MINotify(int p,byte nm[]) {
      pid = p;
      // This line is deprecated
      // name = new String(nm,0);
      // Here's the new version. This may cause a problem as it uses the
      // platform's own encoding.
      name = new String(nm);
   }
}

public class MIStatement implements Statement
{
  static final int MAX_MESSAGE_LEN = 8194;

  MIConnection parent;
  DataInputStream s_in;
  DataOutputStream s_out;

  long results;
  
  int MaxFieldSize = 8192;
  int MaxRows = 0;
  boolean escape = true;
  long QueryTimeout = 0;
  SQLWarning warnings = null;

  // in theory we can have multiple resultsets
  Vector resultSets;
  Vector statuses;
  Vector notifies;

  int currentResult;
  
  boolean BINARY = true;
  
  
  MIStatement(MIConnection newparent) {
    parent = newparent;
    s_in = parent.sock_in;
    s_out = parent.sock_out;
    clearStatus();
  }
  
  private void clearStatus() {
    resultSets = new Vector();
    statuses = new Vector();
    notifies = new Vector();
    currentResult = 0;
  }
  

   private void sendQuery(String sql) throws SQLException {

     //
     // EK: this method is completely rewritten for use with Illustra
     //

     
     // make sure buffer length is > MEX_MESSAGE_LEN in constructor
     // for buffered output stream in MIConnection.java
     int maxlen = MAX_MESSAGE_LEN - 12; 
     if ( sql.length() > maxlen )
       throw new SQLException("Query too long.  Maximum is " +
			      maxlen);
     
     try {
       


       // we may here flush out the last query.....


       // send packet:
       //    Header:
       //       word32 containing length payload = 4 + length(query) + (binary?1:0)
       //       word16 containing channel = 0 (not used)
       //       char8 class 'Q'
       //       char8 subclass 'Q'
       //    Payload:
       //       word32: length of query
       //       chars: query
       //       'B' if binary
              
       int length = 4 + sql.length() + (BINARY?1:0);
       s_out.writeInt(length);
       s_out.writeShort(0);
       s_out.writeByte('Q');
       s_out.writeByte('Q');
       s_out.writeInt(sql.length());
       s_out.writeBytes(sql);
       if (BINARY)
	 s_out.writeByte('B');

       // possible to send Byte 'B' for binary query 
       // and Byte 'P' for precompile.

       s_out.flush(); // important
       
     } catch (Exception e) {
       throw new SQLException("Network write error: "+e);
     }
   } // SendQuery


  // skip next n bytes from input stream
  protected void skipBytesFully(int n) throws SQLException {
    try {
      while (n>0) 
	n -= s_in.skipBytes(n);
    } catch (Exception e) {
      throw new SQLException("Network read error: "+e);
    }
  }

  // read string of n characters
  protected String readstring(int n) throws SQLException {
    
    byte[] b = new byte[n];
    int i = n;

    // read the buffer b
    try {
      while (i>0) 
	i -= s_in.read(b,n-i,i);
    } catch(Exception e) {
      throw new SQLException("Network read error: "+e);
    }

    String s = new String(b);
    return(s);

  }
  

  private void readResults() throws SQLException {
      int type;

      while ( true ) {
         try {


	   // read header
	   //     word32  length
	   //     word16  channel (not used)
	   //     char8   class
	   //     char8   subclass

	   int packetlength = s_in.readInt(); 
	   short channel = s_in.readShort(); 
	   char classid = (char) s_in.readByte(); 
	   char subclassid = (char) s_in.readByte(); 

	   if (MId.debug) 
	     MId.h("readResults, class, subclass = "+classid+","+subclassid);

	   //System.out.println("('"+classid+"','"+subclassid+"')");

	   switch (classid) {

	   case 'Q': 

	     switch (subclassid) {

	     case 'E':  // ('Q','E') indicates an error
	       String errmess;
	       //skipBytesFully(packetlength);
	       if (packetlength > 0) {
		 // try to read errorstring
		 int len = s_in.readInt();
		 errmess = readstring(len);
	       } 
	       else errmess = "?";
	       throw new SQLException("Error: "+errmess);	       
	     
	     
	     case 'A':  // ('Q','A') indicates contract not implemented
	       skipBytesFully(packetlength);
	       throw new SQLException("Contract not implemented");	       
	     

	     case 'R':  // ('Q','R') indicates Ready for qeury
	       skipBytesFully(packetlength);
               //resultSets.addElement(new MIResultSet());	       	       
	       return;
	       //throw new SQLException("Ready for query");	       
	     

	     case 'B':  // ('Q','B') indicates Query Begins
	       skipBytesFully(packetlength);
	       break;  // read next packet
	     

	     case 'C':  // ('Q','C') indicates Query results, done with query
	       // skipBytesFully(packetlength);
	       String mess;
	       if (packetlength > 0) {
		 int len = s_in.readInt();
		 packetlength -= 4;
		 mess = readstring(len);
		 packetlength -= len;
		 int rowsaffected = s_in.readInt();
		 packetlength -= 4;
		 if (MId.debug)
		   System.out.println("('Q','C'):"+mess);
		 if (packetlength>0)
		   skipBytesFully(packetlength);

		 if (mess.startsWith("CREATE") ||
		     mess.startsWith("DROP")) {

		   // DDL statements. 
		   // updateCount should be 0
		   
		   statuses.addElement(new Integer(0));
		   
		 } 
		 else if (mess.startsWith("UPDATE") ||
			  mess.startsWith("INSERT")) {
		   
		   statuses.addElement(new Integer(rowsaffected));

		 } else if (mess.startsWith("SELECT")) {

		   // we don't come here. Is handled in MIResultSet
		   throw new SQLException("Unexpected (Q,C) packet.");

		 }
		   

	       } 
	      
               //resultSets.addElement(new MIResultSet());	       	       
	       break;

	       //return;
	       //throw new SQLException("Query Results");	       
	     

	     case 'D':  // ('Q','D') indicates Datatype Message

	       // get object id
	       long objid = s_in.readLong();
	       packetlength -= 8;   // 64 bit objid
	       
	       // get type name
	       int len = s_in.readInt() - 4;
	       packetlength -= 4;
	       String typename = readstring(len);
	       packetlength -= len;
	       
	       // get type length
	       int typelen = s_in.readInt();
	       packetlength -= 4;

	       // get type info
	       byte type_align = s_in.readByte();
	       byte type_byvalue = s_in.readByte();
	       int type_bound = s_in.readInt();
	       int type_param = s_in.readInt();
	       packetlength -= 10;

	       //System.out.println("** "+typename+" len="+typelen+" bnd="+type_bound+
	       //		  " parm="+type_param);

	       // sometimes there some more stuff:
	       // constructor, subtype,  descriptor for composite type
	       // ignore for now
	       if (packetlength != 0) {
		 if (packetlength>=4) {
		   int constructor = s_in.readInt();
		   //  System.out.println("** constructor="+constructor);
		   packetlength -= 4;
		 }
		 if (packetlength>=4) {
		   int subtype = s_in.readInt();
		   // System.out.println("** subtype="+subtype);
		   packetlength -= 4;
		 }		   
		 if (packetlength > 0)
		   skipBytesFully(packetlength);
	       }
	       
	       // store in type table
	       parent.typetable.addType(objid, typename, typelen);

	       break;
	     

	     case 'F':  // ('Q','F') tuple descriptor
	       // this is handled further by :
               resultSets.addElement(new MIResultSet(this,s_in,packetlength));	       
	       break;

	     case 'T':  // ('Q','T') Tuple message
	       // should not happen here, is handled inside MIResultSet.
	       skipBytesFully(packetlength);
	       throw new SQLException("Tuple Message");	       
	       
	     } // switch (subclassid)
	   } // switch (classid)

	   
         } catch ( SQLException e ) {
            throw e;
         } catch ( Exception e ) {
            throw new SQLException("Network read error: "+e);
         }
      } // while ( true) 
  } // readResults

   // executeQuery method executes the given SQL string which must
   // be a query that returns a single ResultSet.
   // (See also the more general "execute" below.)
   public ResultSet executeQuery(String sql) throws SQLException {
      if ( execute(sql) )          
         return (ResultSet) resultSets.firstElement();
      throw new SQLException("No ResultSet returned");
   }

   // This method execute a SQL statement that is known to be a simple
   // database update (such as a SQL UPDATE, INSERT, DELETE, ...)
   // The result is the number of rows that have been modified.
   // (See also the more general "execute" below.)
   public int executeUpdate(String sql) throws SQLException {
      execute(sql);
      if ( statuses.size() > 0 )
         try {
            return Integer.valueOf((String) statuses.firstElement()).intValue();
         } catch ( Exception e ) {
            return 1;
         }
      throw new SQLException("No Status returned");
   }

   // close frees up internal state associated with the statement.
   // You should normally call close when you are done with a statement.
   // N.B. any ResultSets associated with the Statement will become
   // unusable when the Statement is closed.
   public void close() throws SQLException {
   }

   //----------------------------------------------------------------------

   // The maxFieldSize is a limit (in bytes) on how much data can be
   // returned as part of a field.  If the limit is exceeded, the data
   // is truncated and a warning is added (see Statement.getWarnings).
   // You should normally not need to change the default limit.
   // Zero means no limit.
   public int getMaxFieldSize() throws SQLException {
      return MaxFieldSize;
   }
   public void setMaxFieldSize(int max) throws SQLException {
      MaxFieldSize = max;
   }

   // The maxRows value is a limit on how many rows can be returned as
   // part of a ResultSet.  You should normally not need to change
   // the default limit.  Zero means no limit.
   public int getMaxRows() throws SQLException {
      return MaxRows;
   }
   public void setMaxRows(int max) throws SQLException {
      MaxRows = max;
   }

   // if escape scanning is on (the default) the driver will do escape
   // substitution before sending the SQL to the database.
   public void setEscapeProcessing(boolean enable) throws SQLException {
      escape = enable;
   }

   // The queryTimeout is how many seconds the driver will allow for an
   // SQL statement to execute before giving up.  Zero means no limit.
   public int getQueryTimeout() throws SQLException {
      return (int) (QueryTimeout / 1000);
   }
   public void setQueryTimeout(int seconds) throws SQLException {
      QueryTimeout = seconds * 1000;
   }

   // Cancel can be used by one thread to cancel a statement that
   // is being executed by another thread.
   public void cancel() throws SQLException {

   }

   // getWarnings will return any warning information related to
   // the current statement execution.  Note that SQLWarning may be
   // a chain.  If a statement is re-executed then warnings associated
   // with its previous execution will be automatically discarded.

   public SQLWarning getWarnings() throws SQLException {
      return warnings;
   }
   public void clearWarnings() throws SQLException {
      warnings = null;
   }
   //----------------------- Multiple Results --------------------------

   // Under some (uncommon) situations a single SQL statement may return
   // multiple result sets and/or update counts.  Normally you can ignore
   // this, unless you're executing a stored procedure that you know may
   // return multiple results, or unless you're dynamically executing an
   // unknown SQL string.  The "execute", "getMoreResults", "getResultSet"
   // and "getUpdateCount" methods let you navigate through multiple results.

   // The "execute" method execute a SQL statement and returns "true"
   // if the first result is a ResultSet, false otherwise.  You can
   // then use getResultSet or getUpdateCount to retrieve the result,
   // and getMoreResults to move to any subseuqnet result(s).

   public void exec(String sql) throws SQLException {
      clearStatus();
      sendQuery(sql);
      readResults();
   }

// if ( statuses.size() > 0 ) {
//   int loop=0;
//   while ( loop < statuses.size() )
//      System.out.println("Status: "+statuses.elementAt(loop++));
// } else {
//   System.out.println("statuses.size: <0");
// } 

   public boolean execute(String sql) throws SQLException {
      if ( parent.autoCommit )
         parent.begin();
      exec(sql);
      if ( parent.autoCommit )
         parent.commit();
      if ( resultSets.size() > 0 )
         return true;
      return false;
   }

   // getResultSet returns the current result as a ResultSet.  It
   // returns NULL if the current result is not a ResultSet.
   public ResultSet getResultSet() throws SQLException {
      if ( currentResult < resultSets.size() )
         return (ResultSet) resultSets.elementAt(currentResult);
      return null;
   }

   // getUpdateCount returns the current result, which should be an update
   // count.   It returns -1 if there are no more results or if the
   // current result is a ResultSet.
   public int getUpdateCount() throws SQLException {
      if ( currentResult >= resultSets.size() )
         if ( (currentResult-resultSets.size()) < statuses.size() )
            return ((Integer)statuses.firstElement()).intValue();
      return -1;
   }

   // getMoreResults moves to the next result.  It returns true if
   // this result is a ResultSet.  getMoreResults also implicitly
   // closes the current ResultSet.
   public boolean getMoreResults() throws SQLException {
      if ( currentResult < resultSets.size() ) {
         MIResultSet r = (MIResultSet) resultSets.elementAt(currentResult);
         // r.close();
      }
      if ( ++currentResult >= resultSets.size() )
         return false;
      return true;
   }

   public void setCursorName(String name) throws SQLException
   {

   }

}

